@@ -1,10 +1,10 @@
 package Mojo::Pg::Migrations;
 use Mojo::Base -base;
 
-use Carp qw(croak);
-use Mojo::File qw(path);
+use Carp         qw(croak);
+use Mojo::File   qw(path);
 use Mojo::Loader qw(data_section);
-use Mojo::Util qw(decode);
+use Mojo::Util   qw(decode);
 
 use constant DEBUG => $ENV{MOJO_MIGRATIONS_DEBUG} || 0;
 
@@ -18,6 +18,19 @@ sub from_data {
   return $self->from_string(data_section($class //= caller, $name // $self->name));
 }
 
+sub from_dir {
+  my ($self, $dir) = @_;
+
+  my $migrations = $self->{migrations} = {up => {}, down => {}};
+  for my $file (path($dir)->list_tree({max_depth => 2})->each) {
+    next unless my ($way)     = ($file->basename          =~ /^(up|down)\.sql$/);
+    next unless my ($version) = ($file->dirname->basename =~ /^(\d+)$/);
+    $migrations->{$way}{$version} = decode 'UTF-8', $file->slurp;
+  }
+
+  return $self;
+}
+
 sub from_file { shift->from_string(decode 'UTF-8', path(pop)->slurp) }
 
 sub from_string {
@@ -48,19 +61,26 @@ sub migrate {
 
   # Already the right version (make sure migrations table exists)
   my $db = $self->pg->db;
-  return $self if $self->_active($db, 1) == $target;
+  return $self if $self->_active($db) == $target;
 
   # Lock migrations table and check version again
+  $db->query(
+    'CREATE TABLE IF NOT EXISTS mojo_migrations (
+       name    TEXT PRIMARY KEY,
+       version BIGINT NOT NULL CHECK (version >= 0)
+     )'
+  );
   my $tx = $db->begin;
   $db->query('LOCK TABLE mojo_migrations IN EXCLUSIVE MODE');
-  return $self if (my $active = $self->_active($db, 1)) == $target;
+  return $self if (my $active = $self->_active($db)) == $target;
 
   # Newer version
   croak "Active version $active is greater than the latest version $latest" if $active > $latest;
 
   my $sql = $self->sql_for($active, $target);
   warn "-- Migrate ($active -> $target)\n$sql\n" if DEBUG;
-  $sql .= ';UPDATE mojo_migrations SET version = $1 WHERE name = $2;';
+  $sql .= ';INSERT INTO mojo_migrations (name, version) VALUES ($2, $1)';
+  $sql .= ' ON CONFLICT (name) DO UPDATE SET version = $1;';
   $db->query($sql, $target, $self->name) and $tx->commit;
 
   return $self;
@@ -82,7 +102,7 @@ sub sql_for {
 }
 
 sub _active {
-  my ($self, $db, $create) = @_;
+  my ($self, $db) = @_;
 
   my $name = $self->name;
   my $results;
@@ -91,16 +111,7 @@ sub _active {
     my $sql = 'SELECT version FROM mojo_migrations WHERE name = $1';
     $results = $db->query($sql, $name);
   };
-  if ((my $next = $results->array) || !$create) { return $next->[0] || 0 }
-
-  $db->query(
-    'CREATE TABLE IF NOT EXISTS mojo_migrations (
-       name    TEXT PRIMARY KEY,
-       version BIGINT NOT NULL CHECK (version >= 0)
-     )'
-  ) if $results->sth->err;
-  $db->query('INSERT INTO mojo_migrations VALUES ($1, $2)', $name, 0);
-
+  if (my $next = $results->array) { return $next->[0] || 0 }
   return 0;
 }
 
@@ -186,6 +197,19 @@ the caller class and L</"name">.
   -- 1 down
   DROP TABLE messages;
 
+=head2 from_dir
+
+  $migrations = $migrations->from_dir('/home/sri/migrations');
+
+Extract migrations from a directory tree where each versioned migration is in a directory, named for the version, and
+each migration has one or both of the files named C<up.sql> or C<down.sql>.
+
+  migrations/1/up.sql
+  migrations/1/down.sql
+  migrations/2/up.sql
+  migrations/3/up.sql
+  migrations/3/down.sql
+
 =head2 from_file
 
   $migrations = $migrations->from_file('/home/sri/migrations.sql');
